using System;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace XELF {

	public class HatchingGame : Microsoft.Xna.Framework.Game {
		readonly GraphicsDeviceManager graphics;
		readonly PostEffect postEffect;
		SpriteBatch spriteBatch;
		Model model;
		RenderTarget2D target;
		SpriteFont font;

		bool EnableSourceColor = true;
		bool EnableEffect = true;

		KeyboardState keyboardState;
		KeyboardState previousKeyvboardState;
		StringBuilder s = new StringBuilder();

		public HatchingGame() {
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
			Components.Add(postEffect = new PostEffect(this));
		}

		protected override void Initialize() {
			base.Initialize();
		}
		protected override void LoadContent() {
			spriteBatch = new SpriteBatch(GraphicsDevice);
			font = Content.Load<SpriteFont>("Fonts/Default");

			target = new RenderTarget2D(GraphicsDevice,
				800, 600, 1, SurfaceFormat.Color, RenderTargetUsage.PreserveContents);

			model = Content.Load<Model>("Animal");
		}

		protected override void UnloadContent() {
		}

		public bool IsKeyPress(Keys key) {
			return keyboardState.IsKeyDown(key) && previousKeyvboardState.IsKeyUp(key);
		}

		protected override void Update(GameTime gameTime) {
			//TargetElapsedTime = TimeSpan.FromSeconds(1 / 15.0);
			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
				this.Exit();

			previousKeyvboardState = keyboardState;
			keyboardState = Keyboard.GetState();

			if (IsKeyPress(Keys.F1)) {
				postEffect.Mode = PostEffectMode.Hatch;
			}
			if (IsKeyPress(Keys.B)) {
				postEffect.EffectSensibility -= 1.0f / 16;
			}
			if (IsKeyPress(Keys.N)) {
				postEffect.EffectSensibility += 1.0f / 16;
			}
			if (IsKeyPress(Keys.C)) {
				postEffect.EffectTranslationRange -= new Vector2(1.0f / 16);
			}
			if (IsKeyPress(Keys.V)) {
				postEffect.EffectTranslationRange += new Vector2(1.0f / 16);
			}
			if (IsKeyPress(Keys.Z)) {
				postEffect.EffectAngle -= 1.0f / 16;
			}
			if (IsKeyPress(Keys.X)) {
				postEffect.EffectAngle += 1.0f / 16;
			}
			if (IsKeyPress(Keys.Up)) {
				postEffect.EffectAngleRange -= 1.0f / 16;
			}
			if (IsKeyPress(Keys.Down)) {
				postEffect.EffectAngleRange += 1.0f / 16;
			}
			if (IsKeyPress(Keys.Left)) {
				postEffect.EffectScale--;
			}
			if (IsKeyPress(Keys.Right)) {
				postEffect.EffectScale++;
			}
			if (IsKeyPress(Keys.H)) {
				EnableEffect = !EnableEffect;
				if (EnableEffect) {
					postEffect.EffectIntensity = 1;
				} else {
					postEffect.EffectIntensity = 0;
				}
			}
			if (IsKeyPress(Keys.A)) {
				EnableSourceColor = !EnableSourceColor;
				if (EnableSourceColor) {
					postEffect.AmbientColor = Vector4.One;
				} else {
					postEffect.AmbientColor = Vector4.Zero;
				}
			}

			base.Update(gameTime);
		}

		protected override void Draw(GameTime gameTime) {
			{
				GraphicsDevice.SetRenderTarget(0, target);
				GraphicsDevice.Clear(ClearOptions.Target | ClearOptions.DepthBuffer, Color.White, 1, 0);
				GraphicsDevice.RenderState.CullMode = CullMode.None;
				GraphicsDevice.RenderState.AlphaBlendEnable = false;
				Matrix world = Matrix.CreateRotationY((float)(gameTime.TotalGameTime.TotalSeconds * 0.25f));
				Matrix view = Matrix.CreateLookAt(new Vector3(0, 0, 1000), new Vector3(0, 200, 0), Vector3.Up);
				Matrix projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
					GraphicsDevice.Viewport.AspectRatio, 1, 10000);
				foreach (var mesh in model.Meshes) {
					foreach (BasicEffect effect in mesh.Effects) {
						effect.World = world;
						effect.View = view;
						effect.Projection = projection;
						effect.PreferPerPixelLighting = true;
						effect.EnableDefaultLighting();
						effect.DirectionalLight0.DiffuseColor = new Vector3(2, 2, 2);
					}
					mesh.Draw();
				}
				GraphicsDevice.SetRenderTarget(0, null);

				postEffect.Draw(target.GetTexture());
			}

			{
				spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Texture, SaveStateMode.SaveState);

				s.Remove(0, s.Length);
				s.AppendFormat(
@"AmbientColor={0},
SourceColor={1},
Sensibility={2},
Intensity={3},
Scale={4},
Angle={5},
AngleRange={6},
TranslationRange={7}",
					postEffect.AmbientColor,
					postEffect.SourceColor,
					postEffect.EffectSensibility,
					postEffect.EffectIntensity,
					postEffect.EffectScale,
					postEffect.EffectAngle,
					postEffect.EffectAngleRange,
					postEffect.EffectTranslationRange
					);
				spriteBatch.DrawString(font, s.ToString(), new Vector2(0x20, 0x20), Color.Blue);
				spriteBatch.End();
			}

			base.Draw(gameTime);
		}
	}
}
