float4x4 HatchTransform;

Texture SourceTexture;
Texture HatchTexture;

float2 SourceResolution = float2(800, 600);
float3 HatchResolution = float3(128, 128, 8);
float4 SourceColor = float4(1, 1, 1, 1);
float4 AmbientColor = float4(0, 0, 0, 0);
float4 EffectColor = float4(1, 1, 1, 0);
float EffectSensibility = 1;
float EffectIntensity = 1;
float4 monotone = float4(0.29891, 0.58661, 0.11448, 0);

sampler SourceSampler = sampler_state {
	texture = (SourceTexture);
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

sampler HatchSampler = sampler_state {
	texture = (HatchTexture);
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressW = CLAMP;
};

struct VertexShaderInput {
    float4 Position : POSITION0;
    float4 Location : TEXCOORD0;
};

struct VertexShaderOutput {
    float4 Position : POSITION0;
    float4 Location : TEXCOORD0;
    float4 HatchLocation : TEXCOORD1;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input) {
    VertexShaderOutput output;

    output.Position = input.Position;
	output.Location = input.Location;
	output.HatchLocation = mul(input.Location, HatchTransform);

    return output;
}

float4 HatchPS(VertexShaderOutput input) : COLOR0 {
	float2 hatchLocation = input.HatchLocation;
	float4 source = tex2D(SourceSampler, input.Location + float2(0.5, 0.5) / SourceResolution);
	
	float hatchIntensity = dot(monotone, source) * (7 / 8.0) + (1 - EffectSensibility);

	float3 hatchCoord = float3(hatchLocation, hatchIntensity) + float3(0.5, 0.5, 0.5) / HatchResolution;
	float4 hatch = tex3Dlod(HatchSampler, float4(hatchCoord, 0));

	float4 color = AmbientColor + source * SourceColor;
	color.rgb = lerp(color.rgb, color.rgb * EffectColor.rgb * hatch.rgb, EffectIntensity);
    return color;
}

technique Hatch {
    pass Hatch {
        VertexShader = compile vs_3_0 VertexShaderFunction();
        PixelShader = compile ps_3_0 HatchPS();
    }
}
